#include <Balboa32U4.h>

/*
  This tutorial should show a way to get the actual angle of the Balboa 32U4. 
  Used information: measurements of the gyroscope and accelerometer
  Treatment of initial angle: no treatment yet 

  Madgwick filter: https://github.com/arduino-libraries/MadgwickAHRS
 */
 // starting with some imports
 
#include <MadgwickAHRS.h>
#include <Balboa32U4.h>
#include <Wire.h>
#include <LSM6.h>
float gyro_offsets[3];
float gyro_measure_normalized[3]; 
float acc_measure_normalized[3]; 

// preparation for the Madgwick filter
Madgwick filter;
const float sensorRate = 208.00;

// preparation to print the information
char report[80];

// setup the inertial measurement unit 
LSM6 imu;

// defining some helperfunctions
void initializeIMU()
{ 
  // gyro offset treatment
  float gyro_offset_x=0;
  float gyro_offset_y=0;
  float gyro_offset_z=0;
  // check if the intertial measurement unit (IMU) is initialized
  if (!imu.init())
  {
    Serial.println("Failed to detect and initialize IMU!");
    delay(200);
  }
  // enable the default settings of the IMU
  imu.enableDefault();
  
  // write the accelerometer settings 
  imu.writeReg(LSM6::CTRL1_XL, 0b01011000); // 208 Hz, +/- 4 m/s^2
  // write the gyro settings
  imu.writeReg(LSM6::CTRL2_G, 0b01011000); // 208 Hz, 1000 deg/s
  
  // wait for IMU readings to stabilze
  delay(1000);
  
  // calibrate the gyro 
  for (int i=0; i<1000; i++){
      imu.read();
      gyro_offset_x+=imu.g.x;
      gyro_offset_y+=imu.g.y;
      gyro_offset_z+=imu.g.z;
      delay(1);
    }
  gyro_offset_x = gyro_offset_x/1000;
  gyro_offset_y = gyro_offset_y/1000;
  gyro_offset_z = gyro_offset_z/1000;

  // return the calibration subtrahend 
  gyro_offsets[0] = gyro_offset_x;
  gyro_offsets[1] = gyro_offset_y;
  gyro_offsets[2] = gyro_offset_z;
}

// normalize the input of the gyroscope including the adapt_factor of the data sheet
void normalizedInputGyro(float gyro_x, float gyro_y, float gyro_z, float adapt_factor, float offsets[3]){
  gyro_x = ((float) gyro_x - offsets[0])/adapt_factor; 
  gyro_y = ((float) gyro_y - offsets[1])/adapt_factor;
  gyro_z = ((float) gyro_z - offsets[2])/adapt_factor;
  gyro_measure_normalized[0] = gyro_x;
  gyro_measure_normalized[1] = gyro_y;
  gyro_measure_normalized[2] = gyro_z;
}

// normalize the input of the accelerometer
void normalizedInputAccelerometer(float acc_x, float acc_y, float acc_z, float adapt_factor){
  acc_measure_normalized[0] = acc_x/adapt_factor;
  acc_measure_normalized[1] = acc_y/adapt_factor;
  acc_measure_normalized[2] = acc_z/adapt_factor;
}

// Madgwick filter data processing for the pitch orientation
float pitch_data_process(float pitch, float roll){
  if (abs(roll)>= 90){
    return pitch+90;
  }else{
    return abs(pitch)-90;
  }
}

// run the Arduino script

void setup() {
  
  // starting the communication with the I2C/TWI devices
  Wire.begin();
  
  // initialize the intertial measurement unit (gyroscope and accelerometer)
  initializeIMU();
  filter.begin(sensorRate);
  
}

void loop() {
  // read the input of the imu
  imu.read(); 
  
  // normalize the IMU measurments
  normalizedInputGyro(imu.g.x, imu.g.y, imu.g.z, 28.57, gyro_offsets);
  normalizedInputAccelerometer(imu.a.x, imu.a.y, imu.a.z, 8196.7213);
  
  // values for the orientation of the robot
  float roll, pitch, heading, pitch_data_processed; 
  // run the Madgwick filter 
  filter.updateIMU(gyro_measure_normalized[0], gyro_measure_normalized[1], gyro_measure_normalized[2], acc_measure_normalized[0], acc_measure_normalized[1], acc_measure_normalized[2]);
  // read heading, pitch and roll
  roll = filter.getRoll();
  pitch = filter.getPitch(); 
  heading = filter.getYaw();
  pitch_data_processed = pitch_data_process(pitch, roll);

  // print the heading, pitch and roll 
  Serial.print("Orientation:   heading:");
  Serial.print(heading);
  Serial.print("   pitch:");
  Serial.print(pitch);
  Serial.print("   roll:");
  Serial.print(roll);
  Serial.print("   pitch_data_processed:");
  Serial.println(pitch_data_processed);
  
  
}
